'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var _defineProperty = _interopDefault(require('@babel/runtime/helpers/defineProperty'));
var queryString = require('query-string');

/**
 * Detector classes obtain list of user's locales. It can be from browser, url, cookies, storage, ... whatever you want
 * Get locales method always return array of locales.
 */
class Detector {}

/**
 * Navigator detector try load locales from browser navigator variable.
 */

class NavigatorDetector extends Detector {
  /**
   * On default languages are loaded from window.navigator
   *
   * @param {Navigator} navigator - browser navigator variable
   */
  constructor(navigator = window.navigator) {
    super();

    _defineProperty(this, "navigator", void 0);

    this.navigator = navigator;
  }
  /**
   * Get list of locales.
   *
   * @returns {Array<string>} list of locales from browser
   */


  getLocales() {
    // @ts-ignore
    const {
      languages,
      language,
      userLanguage
    } = this.navigator;

    if (Array.isArray(languages)) {
      return languages;
    }

    if (language) {
      return [language];
    }

    if (userLanguage) {
      return [userLanguage];
    }

    return [];
  }

}

/**
 * This detector load locales from GET parameters. The value should be locale separated list.
 */

class UrlDetector extends Detector {
  /**
   * Constructor
   *
   * @param {string} parameter - Which parameter to look for.
   * @param {Location} location - window.location is used on default
   */
  constructor(parameter, location = window.location) {
    super();

    _defineProperty(this, "location", void 0);

    _defineProperty(this, "parameter", void 0);

    this.location = location;
    this.parameter = parameter;
  }
  /**
   * Get list of locales from browser url.
   *
   * @returns {Array<string>} list of locales
   */


  getLocales() {
    const locales = queryString.parse(this.location.search)[this.parameter] || null;

    if (typeof locales === 'string') {
      return locales.split(',');
    }

    return [];
  }

}

/**
 * With FromListDetector you can set locales by your own.
 */

class FromListDetector extends Detector {
  /**
   * @param {Array<string>} locales - list of locales
   */
  constructor(locales) {
    super();

    _defineProperty(this, "locales", void 0);

    this.locales = locales;
  }
  /**
   * Get list your of locales.
   *
   * @returns {Array<string>} list of your locales
   */


  getLocales() {
    return this.locales;
  }

}

exports.Detector = Detector;
exports.FromListDetector = FromListDetector;
exports.NavigatorDetector = NavigatorDetector;
exports.UrlDetector = UrlDetector;
